@@ -23,14 +23,17 @@
 #include <string.h>
 #include <libxml/HTMLparser.h>
 #include <glib/gi18n.h>
+#include <gst/gst.h>
+#include <gst/pbutils/pbutils.h>
 #include <webkit/webkit.h>
-#include <dbus/dbus-glib.h>
 #include <libsoup/soup-gnome-features.h>
 #include <libsoup/soup-uri.h>
 #include <libsyncdaemon/libsyncdaemon.h>
 #include "oauth.h"
 #include "u1-music-store.h"
 #include "u1-marshal.h"
+#include "u1-codec-installer.h"
+#include "u1-request-chrome.h"
 
 #define U1_MUSIC_LIBRARY_LOCATION  ".ubuntuone/Purchased from Ubuntu One"
 
@@ -47,10 +50,10 @@
 #define U1_CONNECTING_PAGE "connecting.html" 
 #define U1_INITIAL_PAGE "<html><body>Loading Ubuntu One music store</body></html>"
 
+#define DEFAULT_ENCODING "UTF-8"
 #define MAXIMUM_AUTH_RETRIES 5
 
 struct _U1MusicStorePrivate {
-	DBusGConnection *bus;
 	SyncdaemonDaemon *syncdaemon;
 	gchar *base_url;
 
@@ -60,6 +63,23 @@ struct _U1MusicStorePrivate {
 	WebKitWebFrame *frame_to_use;
 	GtkWidget *progress;
 
+	/* Alert bar widgets
+	 * We use two button widgets here, to avoid overlapping conditions
+	 */
+	GtkWidget *alertbar;
+	GtkWidget *alert_label;
+	GtkWidget *subscribe_btn;
+	GtkWidget *install_btn;
+	GtkWidget *install_progress;
+	gboolean folder_subscribed;
+	gboolean codec_installed;
+
+	/* The installer object */
+	U1CodecInstaller *installer;
+
+	/* GStreamer pipeline used to check for the MP3 codec */
+	GstElement *pipeline;
+
 	guint watch_id;
 	guint idle_cb;
 	GHashTable *watched_downloads;
@@ -97,12 +117,12 @@ u1_music_store_finalize (GObject *object)
 		if (music_store->priv->base_url != NULL)
 			g_free (music_store->priv->base_url);
 
-		if (music_store->priv->bus != NULL)
-			dbus_g_connection_unref (music_store->priv->bus);
-
 		if (music_store->priv->syncdaemon != NULL)
 			g_object_unref (G_OBJECT (music_store->priv->syncdaemon));
 
+		if (music_store->priv->installer != NULL)
+			g_object_unref (music_store->priv->installer);
+
 		g_free (music_store->priv);
 	}
 
@@ -237,7 +257,7 @@ get_credentials (U1MusicStore *music_store,
 
 	*oauth_consumer_token = *oauth_consumer_secret = *oauth_token = *oauth_token_secret = NULL;
 
-	/* Get the OAuth token from the keyring */
+	/* Get the OAuth token from the SSO client */
 	if ((credentials = syncdaemon_authentication_find_credentials (
 		     syncdaemon_daemon_get_authentication (music_store->priv->syncdaemon)))) {
 		*oauth_consumer_token = g_strdup (syncdaemon_credentials_get_consumer_key (credentials));
@@ -256,19 +276,9 @@ static gchar *
 get_url_to_use (U1MusicStore *music_store)
 {
 	gchar *oauth_consumer_token, *oauth_consumer_secret, *oauth_token = NULL, *oauth_token_secret = NULL, *real_url;
-	const gchar *url_to_use;
-
-	/* If U1MUSICSTOREURL is defined, use that instead of the real URL */
-	if (! (url_to_use = g_getenv ("U1MUSICSTOREURL")))
-		url_to_use = "https://one.ubuntu.com";
-
-	if (music_store->priv->base_url != NULL)
-		g_free (music_store->priv->base_url);
-
-	music_store->priv->base_url = g_strdup (url_to_use);
 
 	/* OAUth sign the URL */
-	if (g_str_has_prefix (url_to_use, "http://localhost") || g_str_has_prefix (url_to_use, "http://127.0.0.1")) {
+	if (g_str_has_prefix (music_store->priv->base_url, "http://localhost") || g_str_has_prefix (music_store->priv->base_url, "http://127.0.0.1")) {
 		const gchar *oauthfile;
 
 		oauthfile = g_getenv ("OAUTHKEYFILE");
@@ -327,7 +337,8 @@ get_url_to_use (U1MusicStore *music_store)
 static void
 got_new_credentials_cb (SyncdaemonAuthentication *auth, SyncdaemonCredentials *credentials, gpointer user_data)
 {
-	gchar *oauth_consumer_token, *oauth_consumer_secret, *oauth_token, *oauth_token_secret, *url_to_use, *real_url;
+	gchar *oauth_consumer_token, *oauth_consumer_secret, *oauth_token, *oauth_token_secret, *real_url;
+	gchar *url_to_use = NULL;
 	gchar **uri_parts;
 	NotRegisteredReplacementData *nrrd = (NotRegisteredReplacementData *) user_data;
 
@@ -517,8 +528,10 @@ navigation_requested_cb (WebKitWebView *web_view,
 	U1MusicStore *music_store = U1_MUSIC_STORE (user_data);
 	SoupURI *parsed_uri;
 	gchar *internal_uri;
+	gboolean return_val = FALSE;
 
-	g_debug ("navigation requested to %s", webkit_network_request_get_uri (request));
+	/* Hide the alertbar in case we should show something else */
+	gtk_widget_hide (music_store->priv->alertbar);
 
 	/* Remove watching callback for the page */
 	if (music_store->priv->watch_id != 0) {
@@ -558,7 +571,7 @@ navigation_requested_cb (WebKitWebView *web_view,
 			g_debug ("overriding the library page to ours");
 			load_delayed_page (web_view, library_override, policy_decision);
 
-			return TRUE;
+			return_val = TRUE;
 		}
 	} else if (!g_strcmp0 ((const gchar *) parsed_uri->host, "www.7digital.com")) {
 		/* host is 7digital.com. It must be in our store. */
@@ -577,7 +590,7 @@ navigation_requested_cb (WebKitWebView *web_view,
 			g_free (internal_uri);
 			g_free (to_and_from_error_uris);
 
-			return TRUE;
+			return_val = TRUE;
 		}
 	} else if (!g_strcmp0 ((const gchar *) parsed_uri->host, "one.ubuntu.com")) {
 		/* host is one.ubuntu.com. It must not be the login page */
@@ -604,7 +617,7 @@ navigation_requested_cb (WebKitWebView *web_view,
 			g_free (internal_uri);
 			g_free (to_and_from_error_uris);
 
-			return TRUE;
+			return_val = TRUE;
 		}
 	} else if (!g_strcmp0 ((const gchar *) parsed_uri->host, "login.ubuntu.com")) {
 		gchar *real_url;
@@ -629,13 +642,13 @@ navigation_requested_cb (WebKitWebView *web_view,
 		g_free (internal_uri);
 		g_free (to_and_from_error_uris);
 
-		return TRUE;
+		return_val = TRUE;
 	}
 
 	/* Free memory */
 	soup_uri_free (parsed_uri);
 
-	return FALSE;
+	return return_val;
 }
 
 static void
@@ -698,13 +711,13 @@ parse_html_node (U1MusicStore *music_store, WebKitWebView *web_view, htmlNodePtr
 			gint i;
 			SyncdaemonTransferInfo *download;
 
-			prop = xmlGetProp (subnode, "data-path");
+			prop = xmlGetProp (subnode, (xmlChar *) "data-path");
 			if (prop == NULL)
 				continue;
 
 			/* Escape 's in the file name */
 			escaped_prop = g_string_new (NULL);
-			for (i = 0; i < strlen (prop); i++) {
+			for (i = 0; i < strlen ((gchar *) prop); i++) {
 				if (prop[i] == '\'')
 					escaped_prop = g_string_append (escaped_prop, "\\\'");
 				else
@@ -810,7 +823,7 @@ update_status_in_library_page (U1MusicStore *music_store, WebKitWebView *web_vie
 	}
 
 	/* Parse HTML */
-	html_doc = htmlParseDoc (html->str, NULL);
+	html_doc = htmlParseDoc ((xmlChar *) html->str, DEFAULT_ENCODING);
 	root_node = xmlDocGetRootElement (html_doc);
 	parse_html_node (music_store, web_view, root_node, current_downloads);
 
@@ -831,12 +844,28 @@ poll_downloads_cb (gpointer user_data)
 	return TRUE;
 }
 
+static void
+_show_missing_codec_warning (U1MusicStore *music_store)
+{
+	gtk_label_set_text (GTK_LABEL (music_store->priv->alert_label),
+			    _("MP3 playback support is not available. It must be installed to play Previews and Purchased Music on this computer."));
+
+	if (music_store->priv->install_progress)
+		gtk_widget_hide (music_store->priv->install_progress);
+
+	gtk_widget_hide (music_store->priv->subscribe_btn);
+	gtk_widget_show (music_store->priv->install_btn);
+	gtk_widget_show (music_store->priv->alertbar);
+}
+
 static void
 load_finished_cb (WebKitWebView *web_view, WebKitWebFrame *frame, gpointer user_data)
 {
 	const gchar *library_override;
-	gchar *tmp_url;
 	U1MusicStore *music_store = U1_MUSIC_STORE (user_data);
+	SoupURI * uri;
+
+	uri = soup_uri_new (webkit_web_view_get_uri (web_view));
 
 	/* Remove the initial view if still around */
 	if (music_store->priv->initial_web_viewer != NULL) {
@@ -857,8 +886,16 @@ load_finished_cb (WebKitWebView *web_view, WebKitWebFrame *frame, gpointer user_
 		execute_script (web_view, "u1-library-override.js", library_override);
 
 	/* If the URL just loaded is the library page, update download status */
-	tmp_url = g_strdup_printf ("%s%s", music_store->priv->base_url, U1_LIBRARY_URL);
-	if (g_str_has_prefix (webkit_web_view_get_uri (web_view), tmp_url)) {
+	if (g_str_has_prefix (uri->path, U1_LIBRARY_URL)) {
+		/* Show the alert bar if folder is unsubscribed */
+		if (!music_store->priv->folder_subscribed) {
+			gtk_label_set_text (GTK_LABEL (music_store->priv->alert_label),
+					    _("Your 'Purchased Music' folder is not subscribed. New purchases will not download to this computer."));
+			gtk_widget_hide (music_store->priv->install_btn);
+			gtk_widget_show (music_store->priv->subscribe_btn);
+			gtk_widget_show (music_store->priv->alertbar);
+		}
+
 		/* Inject JavaScript to make songs clickable 
 		   Needs to be done before update-status-in-library-page */
 		execute_script (web_view, "u1-songs-clickable.js", NULL);
@@ -868,15 +905,21 @@ load_finished_cb (WebKitWebView *web_view, WebKitWebFrame *frame, gpointer user_
 
 		/* Poll syncdaemon for downloads while the page is loaded */
 		music_store->priv->watch_id = g_timeout_add (1000, (GSourceFunc) poll_downloads_cb, music_store);
+	} else if (g_strcmp0 (uri->path, "/default.aspx") == 0 ||
+		   g_strcmp0 (uri->path, "/stores/default.aspx") == 0) {
+		/* Show the alert bar for codec installation */
+		if (!music_store->priv->codec_installed) {
+			_show_missing_codec_warning (music_store);
+ 		}
 	}
 
-	g_free (tmp_url);
-
 	/* Signal users the URL finished loading */
 	gtk_widget_hide (music_store->priv->progress);
 
 	g_signal_emit (music_store, u1_music_store_signals[URL_LOADED_SIGNAL], 0,
 		       (const gchar *) webkit_web_view_get_uri (WEBKIT_WEB_VIEW (music_store->priv->web_viewer)));
+
+	soup_uri_free (uri);
 }
 
 static void
@@ -887,23 +930,32 @@ status_bar_text_changed_cb (WebKitWebView *web_view, const gchar *text, gpointer
 	if (g_str_has_prefix (text, "u1preview:::")) {
 		gchar **args;
 
+		/* Show the alert bar for codec installation */
+		if (!music_store->priv->codec_installed) {
+			_show_missing_codec_warning (music_store);
+			return;
+ 		}
+
 		args = g_strsplit (text, ":::", 0);
 		if (args != NULL) {
 			g_signal_emit (music_store, u1_music_store_signals[PREVIEW_MP3_SIGNAL], 0, args[1], args[2]);
 			g_strfreev (args);
 		}
-	}
-	if (g_str_has_prefix (text, "u1playlibrary:::")) {
+	} else if (g_str_has_prefix (text, "u1playlibrary:::")) {
 		gchar **args;
 
+		/* Show the alert bar for codec installation */
+		if (!music_store->priv->codec_installed) {
+			_show_missing_codec_warning (music_store);
+			return;
+ 		}
+
 		args = g_strsplit (text, ":::", 2);
 		if (args != NULL) {
 			g_signal_emit (music_store, u1_music_store_signals[PLAY_LIBRARY_SIGNAL], 0, args[1]);
 			g_strfreev (args);
 		}
-	}
-
-	if (g_str_has_prefix (text, "u1showcertificate:::")) {
+	} else if (g_str_has_prefix (text, "u1showcertificate:::")) {
 		/* should show certificate here in its own window. */
 	}
 
@@ -913,10 +965,12 @@ status_bar_text_changed_cb (WebKitWebView *web_view, const gchar *text, gpointer
 static gboolean
 load_error_cb (WebKitWebView *web_view, WebKitWebFrame *frame, const gchar *uri, GError *error, gpointer user_data)
 {
-	U1MusicStore *music_store = U1_MUSIC_STORE (user_data);
-	load_internal_html_page (web_view, U1_DEFAULT_ERROR_PAGE, uri);
+	if (error->domain == WEBKIT_NETWORK_ERROR) {
+		load_internal_html_page (web_view, U1_DEFAULT_ERROR_PAGE, uri);
+		return TRUE;
+	}
 
-	return TRUE;
+	return FALSE;
 }
 
 static gboolean
@@ -955,38 +1009,254 @@ viewer_property_changed_cb (GObject *object, GParamSpec *pspec, gpointer user_da
 	}
 }
 
+static void
+sd_folder_subscribed_cb (SyncdaemonDaemon *daemon, gboolean success,
+			 SyncdaemonFolderInfo *folder_info, gpointer user_data)
+{
+	U1MusicStore *music_store = U1_MUSIC_STORE (user_data);
+	const gchar *new_path = syncdaemon_folder_info_get_path (folder_info);
+	const gchar *lib_path = u1_music_store_get_library_location (music_store);
+
+	if (success && g_strcmp0 (new_path, lib_path) == 0) {
+		music_store->priv->folder_subscribed = TRUE;
+		gtk_widget_hide (music_store->priv->subscribe_btn);
+		gtk_widget_hide (music_store->priv->alertbar);
+	}
+}
+
+static void
+got_gst_element_message (GstBus *bus, GstMessage *msg, gpointer user_data)
+{
+	U1MusicStore *music_store = U1_MUSIC_STORE (user_data);
+	gboolean is_missing = FALSE;
+
+	is_missing = gst_is_missing_plugin_message (msg);
+	if (is_missing) {
+		g_debug ("MP3 playback is missing.");
+		gst_element_set_state (music_store->priv->pipeline,
+				       GST_STATE_NULL);
+		music_store->priv->codec_installed = FALSE;
+	}
+}
+
+static void
+got_gst_eos_message (GstBus *bus, GstMessage *msg, gpointer user_data)
+{
+	U1MusicStore *music_store = U1_MUSIC_STORE (user_data);
+
+	gst_element_set_state (music_store->priv->pipeline, GST_STATE_NULL);
+	if (g_getenv ("U1INSTALLMP3ANYWAY") != NULL)
+		music_store->priv->codec_installed = FALSE;
+	else
+		music_store->priv->codec_installed = TRUE;
+}
+
+static gboolean
+check_mp3_support (gpointer user_data)
+{
+	U1MusicStore *music_store = U1_MUSIC_STORE (user_data);
+	GstBus *bus;
+	GError *error = NULL;
+	gchar *empty_file;
+	gchar *empty_uri;
+	gchar *launch;
+
+	empty_file = g_build_filename (U1_JAVASCRIPT_DIR, "empty.mp3", NULL);
+	empty_uri = g_filename_to_uri (empty_file, NULL, NULL);
+	g_free (empty_file);
+
+	launch = g_strdup_printf ("uridecodebin uri=%s ! fakesink", empty_uri);
+	g_free (empty_uri);
+
+	music_store->priv->pipeline = gst_parse_launch (launch, &error);
+	if (error != NULL) {
+		g_warning ("Error checking for MP3 support: %s",
+			   error->message);
+		music_store->priv->codec_installed = FALSE;
+		return FALSE;
+	}
+	bus = gst_element_get_bus (music_store->priv->pipeline);
+	gst_bus_add_signal_watch (bus);
+	g_signal_connect (G_OBJECT (bus), "message::element",
+			  G_CALLBACK (got_gst_element_message), music_store);
+	g_signal_connect (G_OBJECT (bus), "message::eos",
+			  G_CALLBACK (got_gst_eos_message), music_store);
+	gst_element_set_state (music_store->priv->pipeline, GST_STATE_PLAYING);
+
+	return FALSE;
+}
+
+static void
+_install_codec_error (U1CodecInstaller *installer, GError *error,
+		      gpointer user_data)
+{
+	U1MusicStore *music_store = U1_MUSIC_STORE (user_data);
+
+	music_store->priv->codec_installed = FALSE;
+	if (music_store->priv->install_progress) {
+		gtk_widget_hide (music_store->priv->install_progress);
+	}
+
+	if (music_store->priv->install_btn)
+		gtk_widget_hide (music_store->priv->install_btn);
+
+	if (error->domain == U1_CODEC_INSTALLER_ERROR &&
+	    error->code == U1_CODEC_INSTALLER_ERROR_UNSUPPORTED) {
+		gtk_label_set_text (GTK_LABEL (music_store->priv->alert_label),
+				    _("MP3 codec cannot be installed. Install is only supported on Ubuntu Linux."));
+	} else {
+		_show_missing_codec_warning (music_store);
+	}
+
+	g_error_free (error);
+}
+
+static void
+_install_codec_started (U1CodecInstaller *installer, gpointer user_data)
+{
+	U1MusicStore *music_store = U1_MUSIC_STORE (user_data);
+
+	if (music_store->priv->install_progress == NULL) {
+		music_store->priv->install_progress = u1_codec_installer_get_progress_widget (installer);
+		gtk_box_pack_end (GTK_BOX (music_store->priv->alertbar),
+				  music_store->priv->install_progress,
+				  TRUE, TRUE, 0);
+	}
+
+	gtk_widget_hide (music_store->priv->install_btn);
+	gtk_widget_show (music_store->priv->install_progress);
+}
+
+static void
+_install_codec_finished (U1CodecInstaller *installer, gpointer user_data)
+{
+	U1MusicStore *music_store = U1_MUSIC_STORE (user_data);
+
+	gtk_widget_hide (music_store->priv->alertbar);
+	gtk_widget_hide (music_store->priv->install_btn);
+	gtk_widget_hide (music_store->priv->install_progress);
+
+	music_store->priv->codec_installed = TRUE;
+}
+
+static void
+_install_missing_codec (GtkButton *button, gpointer user_data)
+{
+	U1MusicStore *music_store = U1_MUSIC_STORE (user_data);
+
+	u1_codec_installer_install_codec (music_store->priv->installer,
+					  GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (music_store))));
+}
+
+static void
+subscribe_purchased_folder (GtkButton *button, gpointer user_data)
+{
+	U1MusicStore *music_store = U1_MUSIC_STORE (user_data);
+	SyncdaemonFolderInfo *folder_info;
+
+	folder_info = syncdaemon_daemon_get_folder_info (music_store->priv->syncdaemon,
+							 u1_music_store_get_library_location (music_store));
+	syncdaemon_daemon_subscribe_folder (music_store->priv->syncdaemon,
+					    syncdaemon_folder_info_get_volume_id (folder_info));
+}
+
+static void
+_u1_purchased_enumerate (GObject *source, GAsyncResult *result,
+                         gpointer user_data)
+{
+	U1MusicStore *music_store = U1_MUSIC_STORE (user_data);
+	GFile *dir = G_FILE (source);
+	GFileEnumerator *files;
+	GFileInfo *info;
+	GError *error = NULL;
+
+	files = g_file_enumerate_children_finish (dir, result, &error);
+
+	if (error != NULL) {
+		g_warning ("Error rescanning Purchased Music: %s", error->message);
+		g_error_free (error);
+		return;
+	}
+
+	while ((info = g_file_enumerator_next_file (files, NULL, NULL)) != NULL) {
+		gchar *path;
+
+		path = g_build_filename (g_file_get_path (dir),
+					 g_file_info_get_name (info),
+					 NULL);
+		if (g_file_test (path, G_FILE_TEST_IS_DIR)) {
+			GFile *subdir = g_file_new_for_path (path);
+			g_file_enumerate_children_async (subdir,
+							 G_FILE_ATTRIBUTE_STANDARD_NAME,
+                                        		 G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS,
+                                        		 G_PRIORITY_DEFAULT, NULL,
+                                        		 (GAsyncReadyCallback) _u1_purchased_enumerate,
+                                        		 music_store);
+		} else if (g_file_test (path, G_FILE_TEST_IS_REGULAR)) {
+			g_signal_emit (music_store,
+				       u1_music_store_signals[DOWNLOAD_FINISHED_SIGNAL], 0,
+                                       (const gchar *) path);
+		}
+
+		g_free (path);
+		g_object_unref (info);
+	}
+	g_file_enumerator_close (files, NULL, NULL);
+	g_object_unref (dir);
+}
+
+static void
+_u1_music_store_rescan_purchased_folder (U1MusicStore *music_store)
+{
+	GFile *dir;
+	const gchar *path;
+
+	path = u1_music_store_get_library_location (music_store);
+	dir = g_file_new_for_path (path);
+	g_file_enumerate_children_async (dir,
+					 G_FILE_ATTRIBUTE_STANDARD_NAME,
+					 G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS,
+					 G_PRIORITY_DEFAULT, NULL,
+					 (GAsyncReadyCallback) _u1_purchased_enumerate,
+					 music_store);
+}
+
 static void
 u1_music_store_init (U1MusicStore *music_store)
 {
 	gchar *new_user_agent;
-	GError *error = NULL;
+	const gchar *url_to_use;
+	SoupSession *session;
+	GtkWidget *alert_icon;
+	SyncdaemonFolderInfo *folder_info;
 
 	if (!g_thread_get_initialized ())
 		g_thread_init (NULL);
 
+	gst_init (NULL, NULL);
+
 	music_store->priv = g_new0 (U1MusicStorePrivate, 1);
 	music_store->priv->watched_downloads = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
 	music_store->priv->syncdaemon = syncdaemon_daemon_new ();
 
-	/* Initialize DBus */
-	music_store->priv->bus = dbus_g_bus_get (DBUS_BUS_SESSION, &error);
-	if (error) {
-		g_warning ("Couldn't get session bus: %s", error->message);
-		g_error_free (error);
+	g_signal_connect (G_OBJECT (music_store->priv->syncdaemon),
+			  "folder_subscribed",
+			  G_CALLBACK (sd_folder_subscribed_cb), music_store);
 
-	}
+	/* Set up the installer in case we need it */
+	music_store->priv->installer = u1_codec_installer_new ();
+	g_signal_connect (G_OBJECT (music_store->priv->installer), "error",
+			  G_CALLBACK (_install_codec_error), music_store);
+	g_signal_connect (G_OBJECT (music_store->priv->installer), "started",
+			  G_CALLBACK (_install_codec_started), music_store);
+	g_signal_connect (G_OBJECT (music_store->priv->installer), "finished",
+			  G_CALLBACK (_install_codec_finished), music_store);
 
-	dbus_g_object_register_marshaller (_u1_marshal_VOID__STRING_STRING,
-					   G_TYPE_NONE,
-					   G_TYPE_STRING, G_TYPE_STRING,
-					   G_TYPE_INVALID);
-	dbus_g_object_register_marshaller (g_cclosure_marshal_VOID__VOID,
-					   G_TYPE_NONE,
-					   G_TYPE_INVALID);
-	dbus_g_object_register_marshaller (g_cclosure_marshal_VOID__STRING,
-					   G_TYPE_NONE,
-					   G_TYPE_STRING,
-					   G_TYPE_INVALID);
+	/* If U1MUSICSTOREURL is defined, use that instead of the real URL */
+	if (! (url_to_use = g_getenv ("U1MUSICSTOREURL")))
+		url_to_use = "https://one.ubuntu.com";
+
+	music_store->priv->base_url = g_strdup (url_to_use);
 
 	/* Create web viewer object */
 	music_store->priv->scroll = gtk_scrolled_window_new (NULL, NULL);
@@ -1002,13 +1272,19 @@ u1_music_store_init (U1MusicStore *music_store)
 						  webkit_web_view_get_settings (WEBKIT_WEB_VIEW (music_store->priv->web_viewer))));
 	g_object_set (G_OBJECT (webkit_web_view_get_settings (WEBKIT_WEB_VIEW (music_store->priv->web_viewer))),
 		      "user-agent", new_user_agent,
+		      "default-encoding", DEFAULT_ENCODING,
 		      "enable-default-context-menu", g_getenv ("U1SHOWCONTEXTMENU") != NULL,
 		      "enable-plugins", FALSE,
 		      NULL);
 	g_free(new_user_agent);
 
+	session = webkit_get_default_session ();
+
 	/* Use GNOME proxy settings */
-	soup_session_add_feature_by_type (webkit_get_default_session (), SOUP_TYPE_PROXY_RESOLVER_GNOME);
+	soup_session_add_feature_by_type (session, SOUP_TYPE_PROXY_RESOLVER_GNOME);
+
+	/* Add our handler for u1chrome: URIs */
+	soup_session_add_feature_by_type (session, U1_TYPE_REQUEST_CHROME);
 
 	g_signal_connect (G_OBJECT (music_store->priv->web_viewer), "navigation-policy-decision-requested",
 				  G_CALLBACK (navigation_requested_cb), music_store);
@@ -1022,13 +1298,55 @@ u1_music_store_init (U1MusicStore *music_store)
 	/* Show a temp view when loading the initial store page */
 	music_store->priv->initial_web_viewer = webkit_web_view_new ();
 	g_object_set (G_OBJECT (webkit_web_view_get_settings (WEBKIT_WEB_VIEW (music_store->priv->initial_web_viewer))),
+		      "user-agent", new_user_agent,
+		      "default-encoding", DEFAULT_ENCODING,
 		      "enable-default-context-menu", g_getenv ("U1SHOWCONTEXTMENU") != NULL,
+		      "enable-plugins", FALSE,
 		      NULL);
 	load_internal_html_page (WEBKIT_WEB_VIEW (music_store->priv->initial_web_viewer), 
 	                         U1_CONNECTING_PAGE, NULL);
 	gtk_widget_show (music_store->priv->initial_web_viewer);
 	gtk_container_add (GTK_CONTAINER (music_store->priv->scroll), music_store->priv->initial_web_viewer);
 
+	/* Create the alert bar */
+	music_store->priv->alertbar = gtk_hbox_new (FALSE, 12);
+	alert_icon = gtk_image_new_from_stock (GTK_STOCK_DIALOG_WARNING,
+					       GTK_ICON_SIZE_MENU);
+	gtk_box_pack_start (GTK_BOX (music_store->priv->alertbar), alert_icon,
+			    FALSE, FALSE, 0);
+	gtk_widget_show (alert_icon);
+
+	music_store->priv->alert_label = gtk_label_new ("");
+	gtk_misc_set_alignment (GTK_MISC (music_store->priv->alert_label),
+				0.0, 0.5);
+	gtk_box_pack_start (GTK_BOX (music_store->priv->alertbar),
+			    music_store->priv->alert_label,
+			    TRUE, TRUE, 0);
+	gtk_widget_show (music_store->priv->alert_label);
+
+	/* Button to install MP3 codec if missing */
+	music_store->priv->install_btn = gtk_button_new_with_label (_("Install"));
+	gtk_box_pack_end (GTK_BOX (music_store->priv->alertbar),
+			  music_store->priv->install_btn,
+			  FALSE, FALSE, 6);
+	g_signal_connect (G_OBJECT (music_store->priv->install_btn), "clicked",
+			  G_CALLBACK (_install_missing_codec), music_store);
+
+	/* Check that MP3 codec is available */
+	g_timeout_add_seconds (3, (GSourceFunc) check_mp3_support, music_store);
+
+	/* Button to subscribe the Purchased Music folder */
+	music_store->priv->subscribe_btn = gtk_button_new_with_label (_("Subscribe"));
+	gtk_box_pack_end (GTK_BOX (music_store->priv->alertbar),
+			  music_store->priv->subscribe_btn,
+			  FALSE, FALSE, 6);
+	g_signal_connect (G_OBJECT (music_store->priv->subscribe_btn), "clicked",
+			  G_CALLBACK (subscribe_purchased_folder), music_store);
+
+	/* Figure out if the Purchased Music folder is subscribed */
+	folder_info = syncdaemon_daemon_get_folder_info (music_store->priv->syncdaemon, u1_music_store_get_library_location (music_store));
+	music_store->priv->folder_subscribed = syncdaemon_folder_info_get_subscribed (folder_info);
+
 	/* Create a hidden progress bar */
 	music_store->priv->progress = gtk_progress_bar_new ();
 	
@@ -1038,8 +1356,11 @@ u1_music_store_init (U1MusicStore *music_store)
 	/* And load the real store in the background */
 	music_store->priv->idle_cb = g_idle_add ((GSourceFunc) load_real_store_cb, music_store);
 
-	gtk_box_pack_start (GTK_BOX (music_store), music_store->priv->scroll, TRUE, TRUE, 3);
-	gtk_box_pack_start (GTK_BOX (music_store), music_store->priv->progress, FALSE, FALSE, 3);
+	gtk_box_pack_start (GTK_BOX (music_store), music_store->priv->alertbar, FALSE, FALSE, 6);
+	gtk_box_pack_start (GTK_BOX (music_store), music_store->priv->scroll, TRUE, TRUE, 0);
+	gtk_box_pack_end (GTK_BOX (music_store), music_store->priv->progress, FALSE, FALSE, 0);
+
+	_u1_music_store_rescan_purchased_folder (music_store);
 }
 
 /**
@@ -1098,6 +1419,12 @@ u1_music_store_load_store_link (U1MusicStore *music_store, const gchar *url)
 	g_return_if_fail (U1_IS_MUSIC_STORE (music_store));
 	g_return_if_fail (url != NULL);
 
+	/* If the load_real_store callback is set up, disable it first */
+	if (music_store->priv->idle_cb > 0) {
+		g_source_remove (music_store->priv->idle_cb);
+		music_store->priv->idle_cb = 0;
+	}
+
 	real_url = g_strdup_printf ("%s%s?forward_on_to_url=%s",
 				    music_store->priv->base_url,
 				    U1_STORE_URL,
